// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/Beam.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/ChargedFinalState.hh"
#include "Rivet/Projections/InitialQuarks.hh"
#include "Rivet/Projections/Thrust.hh"
#include "Rivet/Projections/ParisiTensor.hh"
#include "Rivet/Projections/Hemispheres.hh"

namespace Rivet {


  class L3_2004_I652683 : public Analysis {
  public:

    /// Constructor
    L3_2004_I652683()
      : Analysis("L3_2004_I652683"), _SumOfWeights_udsc(0.), _SumOfWeights_b(0.), _SumOfWeightsCharged(0.), _SumOfWeightsCharged_udsc(0.), _SumOfWeightsCharged_b(0.)
    {    }


    
    /// Book histograms and initialise projections before the run
    void init() {
      // Projections to use
      const FinalState FS;
      addProjection(FS, "FS");
      addProjection(Beam(), "beams");
      const ChargedFinalState CFS;
      addProjection(CFS, "CFS");
      const Thrust thrust(FS);
      addProjection(thrust, "thrust");
      addProjection(ParisiTensor(FS), "Parisi");
      addProjection(Hemispheres(thrust), "Hemispheres");
      addProjection(InitialQuarks(), "initialquarks");

      // Book the histograms
      _h_Thrust_udsc             = bookHisto1D(47, 1, 1);
      _h_Thrust_bottom           = bookHisto1D(47, 1, 2);
      _h_heavyJetmass_udsc       = bookHisto1D(48, 1, 1);
      _h_heavyJetmass_bottom     = bookHisto1D(48, 1, 2);
      _h_totalJetbroad_udsc      = bookHisto1D(49, 1, 1);
      _h_totalJetbroad_bottom    = bookHisto1D(49, 1, 2);
      _h_wideJetbroad_udsc       = bookHisto1D(50, 1, 1);
      _h_wideJetbroad_bottom     = bookHisto1D(50, 1, 2);
      _h_Cparameter_udsc         = bookHisto1D(51, 1, 1); 
      _h_Cparameter_bottom       = bookHisto1D(51, 1, 2);
      _h_Dparameter_udsc         = bookHisto1D(52, 1, 1);
      _h_Dparameter_bottom       = bookHisto1D(52, 1, 2);
      _h_Ncharged                = bookHisto1D(59, 1, 1);
      _h_Ncharged_udsc           = bookHisto1D(59, 1, 2);
      _h_Ncharged_bottom         = bookHisto1D(59, 1, 3);
      _h_scaledMomentum          = bookHisto1D(65, 1, 1);
      _h_scaledMomentum_udsc     = bookHisto1D(65, 1, 2);
      _h_scaledMomentum_bottom   = bookHisto1D(65, 1, 3);

    }


    /// Perform the per-event analysis
    void analyze(const Event& event) {
      const double weight = event.weight();

      // Get beam average momentum
      const ParticlePair& beams = applyProjection<Beam>(event, "beams").beams();
      const double BeamMomentum = ( beams.first.p3().mod() + beams.second.p3().mod() ) / 2.0;

      // InitialQuarks projection to have udsc events separated from b events
      int flavour = 0;
      const InitialQuarks& IQF = applyProjection<InitialQuarks>(event, "initialquarks");
      Particles quarks;
      if ( IQF.particles().size() == 2 ) {
	flavour = IQF.particles().front().abspid();
	quarks  = IQF.particles();
      }
      else {
	map<int, Particle> quarkmap;
	foreach ( const Particle& p, IQF.particles()) {
	  if (quarkmap.find(p.pid()) == quarkmap.end()) quarkmap[p.pid()] = p;
          else if (quarkmap[p.pid()].E() < p.E()) quarkmap[p.pid()] = p;
        }
        double Max_Energy = 0.;
        for (int i = 1; i <= 5; ++i) {
          double energy = 0.;
          if (quarkmap.find( i) != quarkmap.end())
            energy += quarkmap[ i].E();
          if (quarkmap.find(-i) != quarkmap.end())
            energy += quarkmap[-i].E();
          if (energy > Max_Energy)
            flavour = i;
        }
        if (quarkmap.find(flavour) != quarkmap.end())
          quarks.push_back(quarkmap[flavour]);
        if (quarkmap.find(-flavour) != quarkmap.end())
          quarks.push_back(quarkmap[-flavour]);
      }

      switch (flavour) {
      case PID::DQUARK:
      case PID::UQUARK:
      case PID::SQUARK:
      case PID::CQUARK:
      _SumOfWeights_udsc += weight;
      break;
      case PID::BQUARK:
      _SumOfWeights_b += weight;
      break;
      }

      const FinalState& cfs = applyProjection<FinalState>(event, "CFS");
      //      std::cout << "Charged multiplicity -=== " << cfs.size() << std::endl;      
      // Charged Multiplicity
        _SumOfWeightsCharged += weight;
        _h_Ncharged->fill(cfs.size(), weight);
      switch (flavour) {
      case PID::DQUARK:
      case PID::UQUARK:
      case PID::SQUARK:
      case PID::CQUARK:
	 _SumOfWeightsCharged_udsc += weight;
         _h_Ncharged_udsc->fill(cfs.size(), weight);
      break;
      case PID::BQUARK:
        _SumOfWeightsCharged_b += weight;
	_h_Ncharged_bottom->fill(cfs.size(), weight);
      break;
      }
      
      // Iterate over charged particles and get momentum
      foreach (const Particle& p, cfs.particles()) {
	const Vector3 momentum3 = p.p3();
	const double mom = momentum3.mod();
	const double scaledMom = mom/BeamMomentum;
	  _h_scaledMomentum->fill(-std::log(scaledMom), weight);
	switch (flavour) {
	case PID::DQUARK:
	case PID::UQUARK:
	case PID::SQUARK:
	case PID::CQUARK:
	_h_scaledMomentum_udsc->fill(-std::log(scaledMom), weight);
	break;
	case PID::BQUARK:
       	_h_scaledMomentum_bottom->fill(-std::log(scaledMom), weight);
	break;
	}
      }


      const Thrust& thrust = applyProjection<Thrust>(event, "thrust");
      switch (flavour) {
      case PID::DQUARK:
      case PID::UQUARK:
      case PID::SQUARK:
      case PID::CQUARK:
      _h_Thrust_udsc->fill(thrust.thrust(), weight);
      break;
      case PID::BQUARK:
      _h_Thrust_bottom->fill(thrust.thrust(), weight);
      break;
      }

      // C and D Parisi parameters
      const ParisiTensor& parisi = applyProjection<ParisiTensor>(event, "Parisi");
      switch (flavour) {
      case PID::DQUARK:
      case PID::UQUARK:
      case PID::SQUARK:
      case PID::CQUARK:
      _h_Cparameter_udsc->fill(parisi.C(), weight);
      _h_Dparameter_udsc->fill(parisi.D(), weight);
      break;
      case PID::BQUARK:
      _h_Cparameter_bottom->fill(parisi.C(), weight);
      _h_Dparameter_bottom->fill(parisi.D(), weight);
      break;
      }

      // The hemisphere variables
      const Hemispheres& hemisphere = applyProjection<Hemispheres>(event, "Hemispheres");
      switch (flavour) {
      case PID::DQUARK:
      case PID::UQUARK:
      case PID::SQUARK:
      case PID::CQUARK:
      _h_heavyJetmass_udsc->fill(hemisphere.scaledM2high(), weight);
      _h_totalJetbroad_udsc->fill(hemisphere.Bsum(), weight);
      _h_wideJetbroad_udsc->fill(hemisphere.Bmax(), weight);
      break;
      case PID::BQUARK:
      _h_heavyJetmass_bottom->fill(hemisphere.scaledM2high(), weight);
      _h_totalJetbroad_bottom->fill(hemisphere.Bsum(), weight);
      _h_wideJetbroad_bottom->fill(hemisphere.Bmax(), weight);
      break;
      }
      
    }


    /// Normalise histograms etc., after the run
    void finalize() {
      
      scale(_h_Thrust_udsc, 1./_SumOfWeights_udsc);  
      scale(_h_Thrust_bottom, 1./_SumOfWeights_b);
      scale(_h_heavyJetmass_udsc, 1./_SumOfWeights_udsc);
      scale(_h_heavyJetmass_bottom, 1./_SumOfWeights_b);
      scale(_h_totalJetbroad_udsc, 1./_SumOfWeights_udsc);  
      scale(_h_totalJetbroad_bottom, 1./_SumOfWeights_b);
      scale(_h_wideJetbroad_udsc, 1./_SumOfWeights_udsc);
      scale(_h_wideJetbroad_bottom, 1./_SumOfWeights_b); 
      scale(_h_Cparameter_udsc, 1./_SumOfWeights_udsc);     
      scale(_h_Cparameter_bottom, 1./_SumOfWeights_b);   
      scale(_h_Dparameter_udsc, 1./_SumOfWeights_udsc);     
      scale(_h_Dparameter_bottom, 1./_SumOfWeights_b);   
      scale(_h_Ncharged, 2.0/_SumOfWeightsCharged);            
      scale(_h_Ncharged_udsc, 2./_SumOfWeightsCharged_udsc);       
      scale(_h_Ncharged_bottom, 2./_SumOfWeightsCharged_b);     
      scale(_h_scaledMomentum, 1./_SumOfWeightsCharged);      
      scale(_h_scaledMomentum_udsc, 1./_SumOfWeightsCharged_udsc); 
      scale(_h_scaledMomentum_bottom, 1./_SumOfWeightsCharged_b);     
    }

   
  private:
    double _SumOfWeights_udsc, _SumOfWeights_b;
    double _SumOfWeightsCharged, _SumOfWeightsCharged_udsc, _SumOfWeightsCharged_b;
    Histo1DPtr _h_Thrust_udsc;
    Histo1DPtr _h_Thrust_bottom;
    Histo1DPtr _h_heavyJetmass_udsc;
    Histo1DPtr _h_heavyJetmass_bottom;
    Histo1DPtr _h_totalJetbroad_udsc; 
    Histo1DPtr _h_totalJetbroad_bottom;
    Histo1DPtr _h_wideJetbroad_udsc;
    Histo1DPtr _h_wideJetbroad_bottom; 
    Histo1DPtr _h_Cparameter_udsc;     
    Histo1DPtr _h_Cparameter_bottom;   
    Histo1DPtr _h_Dparameter_udsc;     
    Histo1DPtr _h_Dparameter_bottom;   
    Histo1DPtr _h_Ncharged;            
    Histo1DPtr _h_Ncharged_udsc;       
    Histo1DPtr _h_Ncharged_bottom;     
    Histo1DPtr _h_scaledMomentum;      
    Histo1DPtr _h_scaledMomentum_udsc; 
    Histo1DPtr _h_scaledMomentum_bottom;

  };



  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(L3_2004_I652683);


}
